package core

import (
	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/token"
	"git.akyoto.dev/cli/q/src/types"
)

// ExpressionsToRegisters moves multiple expressions into the specified registers and checks that the types match with the function signature.
func (f *Function) ExpressionsToRegisters(expressions []*expression.Expression, registers []cpu.Register, params []*Parameter, alwaysSave bool) error {
	for i := len(expressions) - 1; i >= 0; i-- {
		if alwaysSave {
			f.SaveRegister(registers[i])
		} else {
			for j := i - 1; j >= 0; j-- {
				if f.UsesRegister(expressions[j], registers[i]) {
					f.SaveRegister(registers[i])
					break
				}
			}
		}

		typ, err := f.ExpressionToRegister(expressions[i], registers[i])

		if err != nil {
			return err
		}

		if params == nil {
			continue
		}

		if !types.Is(typ, params[i].Type()) {
			_, expectsPointer := params[i].Type().(*types.Pointer)

			if expectsPointer && expressions[i].Token.Kind == token.Number && expressions[i].Token.Text(f.File.Bytes) == "0" {
				continue
			}

			if f.Package == "mem" && f.Name == "alloc" {
				continue
			}

			return errors.New(&errors.TypeMismatch{Encountered: typ.Name(), Expected: params[i].Type().Name(), ParameterName: params[i].Name()}, f.File, expressions[i].Token.Position)
		}
	}

	return nil
}
